/**
 * @author borning
 *
 * An array-like class.  The purpose of class is to illustrate using inner
 * classes to implement iterators (if you were actually writing Java code,
 * you'd just use Java arrays).
 */

import java.util.Iterator;
import java.util.NoSuchElementException;

public class MyArray {
	/* internal array to actually hold the data */
	private Object[] internalArray;

	/**
	 * the constructor
	 */
	public MyArray(int n) {
		internalArray = new Object[n];
	}

	/**
	 * return length of the array
	 */
	public int length() {
		return internalArray.length;
	}

	/**
	 * get an element
	 */
	public Object at(int i) {
		return internalArray[i];
	}

	/**
	 * set an element
	 */
	public void set(int i, Object value) {
		internalArray[i] = value;
	}

	/**
	 * inner class for the iterator
	 */
	class MyIterator implements Iterator {
		int index;
		MyIterator() {
			index = 0;
		}
		public Object next() {
			Object temp;
			if (!hasNext())
				throw new NoSuchElementException("no next element available");
			temp = internalArray[index];
			index++;
			return temp;
		}
		public boolean hasNext() {
			return index < internalArray.length;
		}
		public void remove() {
			/* this is an optional operation - we don't support it */
			throw new UnsupportedOperationException("remove operation not supported");
		}
	}

	/**
	 * return an iterator for this array
	 */
	public Iterator iterator() {
		return new MyIterator();
	}

}
