import java.util.Iterator;
import java.util.NoSuchElementException;
import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * A JUnit test class to test the MyArray demonstration class
 * @author borning
  */
public class MyArrayTest extends TestCase {

	/* fields for some test arrays */
	private MyArray empty_array;
	private MyArray little_array;
	private static final int little_arraysize = 3;

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		empty_array = new MyArray(0);
		little_array = new MyArray(little_arraysize);
		little_array.set(0, "first");
		little_array.set(1, "second");
		little_array.set(2, "third");
	}

	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
		empty_array = null;
		little_array = null;
	}

	public void testMyArray() {
		MyArray a = new MyArray(0);
		Assert.assertEquals(a.length(), 0);
		MyArray b = new MyArray(100);
		Assert.assertEquals(b.length(), 100);
	}

	public void testLength() {
		Assert.assertEquals(empty_array.length(), 0);
		Assert.assertEquals(little_array.length(), little_arraysize);
	}

	public void testAt() {
		Assert.assertEquals(little_array.at(0), "first");
		Assert.assertEquals(little_array.at(1), "second");
		Assert.assertEquals(little_array.at(2), "third");
	}

	public void testSet() {
		/* the set method is already tested by the setUp and testAt methods ... 
		 * but we'll be redundant and test it here also */
		MyArray b = new MyArray(1);
		Integer testint = new Integer(42);
		b.set(0, testint);
		Assert.assertEquals(b.at(0), testint);
	}

	public void testIterator() {
		/* this test tests both the iterator() method for MyArray, and also the 3 methods that
		 * the iterator provides, namely hasNext(), next(), and remove() */

		/* for the empty array, the iterator should immediately say it doesn't have any more elements */
		Iterator i = empty_array.iterator();
		Assert.assertFalse(i.hasNext());
		/* also, invoking next() should raise an exception */
		iteratorNoNext(i);
		
		/* get all the elements out of the little array, make sure they are correct, 
		 * and that the iterator correctly says when it has more elements */
		Iterator j = little_array.iterator();
		Assert.assertTrue(j.hasNext());
		Assert.assertEquals(j.next(), "first");
		Assert.assertTrue(j.hasNext());
		Assert.assertEquals(j.next(), "second");
		Assert.assertTrue(j.hasNext());
		Assert.assertEquals(j.next(), "third");
		Assert.assertFalse(j.hasNext());
		iteratorNoNext(j);

		/* test that the iterator's remove() method is correct (with a separate test method) */
		iteratorRemoveTest(j);
	}

	private void iteratorRemoveTest(Iterator k) {
		/* in MyArray, we don't support remove().  The spec for this says that it 
		 * should raise an exception if remove() is invoked - so the test makes
		 * sure this exception is properly raised.
		 */
		boolean caught = false;
		try {
			k.remove();
		} catch (UnsupportedOperationException e) {
			caught = true;
		}
		Assert.assertTrue(caught);
	}
	private void iteratorNoNext(Iterator k) {
		/* if we've reached the end with an iterator, but still ask for the next element,
		 * we should raise an exception
		 */
		boolean caught = false;
		try {
			k.next();
		} catch (NoSuchElementException e) {
			caught = true;
		}
		Assert.assertTrue(caught);
	}
}
